@@ -6,8 +6,8 @@ from .policy import ACKTRTorchPolicy
 from .policy import DEFAULT_OPTIM_CONFIG
 
 
-@trainer.config("torch_optimizer", DEFAULT_OPTIM_CONFIG, override=True)
-@TRPOTrainer.with_base_specs
+@trainer.configure
+@trainer.option("torch_optimizer", DEFAULT_OPTIM_CONFIG, override=True)
 class ACKTRTrainer(TRPOTrainer):
     """Single agent trainer for ACKTR."""
 
